/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* [L4D2] Special Infected Warnings Vocalize Fix
* 
* About : This plugin aims to be a simple fix for some
*  special infected warnings never being called upon.
* 
* =============================
* ===      Change Log       ===
* =============================
* Version 1.0    2014-08-24
* - Initial Release
* =============================
* Version 1.1    2014-08-25
* - Added a cvar that allows to change the chance of a vocalization warning
* =============================
* Version 1.2	 2014-08-25
* - Added HeardBoomer, HeardSmoker and HeardTank support for L4D1 Survivors
* =============================
* Version 1.3	 2014-08-26 (50+ views)
* - Added new CVARs to control the delays for how much a survivor must wait
*   being able to vocalize for hearing the same special again, and another for
*   how much a survivor should wait between different specials
* =============================
* Version 1.4    2014-09-11
* - Added a check to see if a survivor isn't already vocalizing before
*   attempting to warn for an infected, as nt doing so would cause the
*   vocalization to fail but still run the timer to delay for the next
*   infected warning.
* =============================
* Version 1.4.1 2014-09-28 (by id5473)
* - Removed all Timers/conditions and exchanged vcd reference for a talker concept.
*   Using context in talkerfiles to apply timers and conditions to the world. (possible gain: unknown :p)
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


#include <sourcemod>
#include <sdktools>
#include <sceneprocessor> 

#define MODEL_NICK "models/survivors/survivor_gambler.mdl"
#define MODEL_ROCHELLE "models/survivors/survivor_producer.mdl"
#define MODEL_COACH "models/survivors/survivor_coach.mdl"
#define MODEL_ELLIS "models/survivors/survivor_mechanic.mdl"

#define MODEL_BILL "models/survivors/survivor_namvet.mdl"
#define MODEL_FRANCIS "models/survivors/survivor_biker.mdl"
#define MODEL_LOUIS "models/survivors/survivor_manager.mdl"
#define MODEL_ZOEY "models/survivors/survivor_teenangst.mdl"
#define WARN_BOOMER "WarnBoomer"
#define WARN_SMOKER "WarnSmoker"
#define WARN_SPITTER "WarnSpitter"
#define WARN_CHARGER "WarnCharger"
#define WARN_TANK "WarnTank"

#define ZOMBIECLASS_SMOKER 1
#define ZOMBIECLASS_BOOMER 2
#define ZOMBIECLASS_HUNTER 3
#define ZOMBIECLASS_SPITTER 4
#define ZOMBIECLASS_JOCKEY 5
#define ZOMBIECLASS_CHARGER 6
#define ZOMBIECLASS_TANK 8

#define PICK_COACH 11
#define PICK_NICK 12
#define PICK_ELLIS 13
#define PICK_ROCHELLE 14
#define PICK_LOUIS 15
#define PICK_FRANCIS 16
#define PICK_BILL 17
#define PICK_ZOEY 18

#define PLUGIN_VERSION "1.2"

public Plugin:myinfo =
{
	name = "[L4D2] Special Infected Warnings Vocalize Fix",
	author = "DeathChaos25",
	description = "Fixes the 'I heard a (Insert Special Infected here)' warning lines not working for some specific Special Infected.",
	version = PLUGIN_VERSION,
	url = "https://forums.alliedmods.net/showthread.php?p=2189049",
}

public OnPluginStart() 
{ 
	HookEvent("player_spawn", PlayerSpawn_Event) 
	CreateConVar("l4d2_si_warnings_vocalize_fix_version", PLUGIN_VERSION, "[L4D2] Special Infected Warnings Vocalize Fix", FCVAR_PLUGIN|FCVAR_SPONLY|FCVAR_REPLICATED|FCVAR_NOTIFY|FCVAR_DONTRECORD) 

}

public PlayerSpawn_Event(Handle:event, const String:name[], bool:dontBroadcast) 
{
	new client = GetClientOfUserId(GetEventInt(event, "userid"));
	
	if (client <= 0 || client > MaxClients || !IsClientInGame(client) || GetClientTeam(client) != 3) {
		return
	}
	decl String:model[PLATFORM_MAX_PATH] ;
	decl String:announcer_model[PLATFORM_MAX_PATH] ;
	decl String:announce_special[PLATFORM_MAX_PATH] ;
	
	new class = GetEntProp(client, Prop_Send, "m_zombieClass");
	
	new i, y, announcer
	new maxplayers = GetMaxClients();
	
	for (i = 1; i <= maxplayers; i++)
	{
		if (IsClientInGame(i)) {
			if (GetClientTeam(i) == 2 && IsPlayerAlive(i)) {
				if (IsActorBusy(i) == true) {
					return
				}
			}
		}
	}
	
	//5 attempts to pick a survivor who's actually there
	for (i = 1; i <= 5; i++) {
	
		//pick character who'll announce infected
		announcer = GetRandomInt(11, 18);
		
		if (announcer == PICK_COACH)  announcer_model = MODEL_COACH 
		else if (announcer == PICK_NICK)  announcer_model = MODEL_NICK
		else if (announcer == PICK_ELLIS)  announcer_model = MODEL_ELLIS 
		else if (announcer == PICK_ROCHELLE) announcer_model = MODEL_ROCHELLE 
		else if (announcer == PICK_LOUIS)  announcer_model = MODEL_LOUIS 
		else if (announcer == PICK_FRANCIS)  announcer_model = MODEL_FRANCIS 
		else if (announcer == PICK_BILL) announcer_model = MODEL_BILL
		else if (announcer == PICK_ZOEY)  announcer_model = MODEL_ZOEY 
	
		//fill in talker concept based on zombieclass
		if (class == ZOMBIECLASS_BOOMER)  announce_special = WARN_BOOMER 
		else if (class == ZOMBIECLASS_SMOKER) announce_special = WARN_SMOKER 
		else if (class == ZOMBIECLASS_CHARGER) announce_special = WARN_CHARGER 
		else if (class == ZOMBIECLASS_SPITTER) announce_special = WARN_SPITTER 
		else if (class == ZOMBIECLASS_TANK) announce_special = WARN_TANK
	
		//determine Playerindex <-> Character and then vocalize
		for (y = 1; y <= maxplayers; y++)
		{
			if (IsClientInGame(y)) {
			
				GetClientModel(i, model, sizeof(model))
				if (StrEqual(model, announcer_model)) {
					PerformSceneEx(y, announce_special, _, 2.0, 1.0)
					i=5; //no more attempts needed to check if survivor is there - check previous loop limit 
					break //breaking out of subloop
				}
			}
		}	
	}	
}
